<?php

/*
Copyright (C) 2016-2017 Pimwick, LLC

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

defined( 'ABSPATH' ) or exit;

global $pw_gift_cards;
global $wpdb;

require( 'header.php' );

$active_count = 0;
$outstanding_balance = 0;

$results = $wpdb->get_row( "
    SELECT
        COUNT(DISTINCT card.pimwick_gift_card_id) AS active_count,
        SUM(activity.amount) AS outstanding_balance
    FROM
        {$wpdb->pimwick_gift_card} AS card
    JOIN
        {$wpdb->pimwick_gift_card_activity} AS activity ON (activity.pimwick_gift_card_id = card.pimwick_gift_card_id)
    WHERE
        card.active = 1
        AND (card.expiration_date IS NULL OR card.expiration_date >= NOW())
    ORDER BY
        card.create_date
" );
if ( null !== $results ) {
    $active_count = $results->active_count;
    $outstanding_balance = $results->outstanding_balance;
}

?>
<div class="pwgc-main-content">

    <?php
        $gift_card_product = $pw_gift_cards->get_gift_card_product();

        ?>
        <div id="pwgc-setup-container">
            <div id="pwgc-setup-error"></div>
            <div class="pwgc-setup-section">
                <div class="pwgc-setup-header">
                    <?php printf( __( 'Step 1: Create a "%s" product to start selling Gift Cards.', 'pimwick' ), PWGC_PRODUCT_TYPE_NAME ); ?>
                </div>
                <?php
                    if ( empty( $gift_card_product ) ) {
                        $create_product_button_text = __( 'Create the Gift Card product', 'pimwick' );

                        ?>
                        <a href="#" id="pwgc-setup-create-product" class="button button-secondary" data-text="<?php echo esc_attr( $create_product_button_text ); ?>"><?php echo esc_attr( $create_product_button_text ); ?></a>
                        <?php
                    }
                ?>
                <div id="pwgc-setup-create-product-success" style="<?php echo empty( $gift_card_product ) ? '' : 'display: block;'; ?>">
                    <div class="pwgc-setup-header" style="color: green;"><i class="fas fa-check-circle"></i> <?php _e( 'Success!', 'pimwick' ); ?></div>
                    <?php _e( 'The Gift Card product has been created. Click on the Products menu in the left to edit it.', 'pimwick' ); ?>
                </div>
            </div>
            <div class="pwgc-setup-section">
                <div class="pwgc-setup-header">
                    <?php _e( 'Step 2: Let your customers check their gift card balances.', 'pimwick' ); ?>
                </div>
                <?php printf( __( 'Upgrade to <a href="%s" target="_blank">PW WooCommerce Gift Cards Pro</a> to create a Check Balance page for your customers.', 'pimwick' ), 'https://www.pimwick.com/gift-cards/' ); ?></a>
            </div>
        </div>
        <?php
    ?>

    <div class="pwgc-summary-container">
        <div class="pwgc-summary-item">
            <div class="pwgc-summary-item-header"><?php echo number_format( $active_count ); ?></div>
            <div><?php _e( 'Active gift cards', 'pimwick' ); ?></div>
        </div>
        <div class="pwgc-summary-item">
            <div class="pwgc-summary-item-header"><?php echo wc_price( $outstanding_balance ); ?></div>
            <div><?php _e( 'Outstanding balances', 'pimwick' ); ?></div>
        </div>
    </div>

    <form id="pwgc-balance-form">
        <div id="pwgc-balance-container">
            <div id="pwgc-balance-number-container">
                <input type="text" id="pwgc-balance-number" name="card_number" autocomplete="off" placeholder="<?php _e( 'Gift Card Number', 'pimwick' ); ?>" value="<?php echo isset( $_GET['card_number'] ) ? esc_html( $_GET['card_number'] ) : ''; ?>" required>
                <input type="submit" class="button button-primary" value="<?php _e( 'View Gift Card Activity', 'pimwick' ); ?>">
            </div>

            <div id="pwgc-balance-error"></div>

            <div id="pwgc-results-container">
                <div id="pwgc-balance-amount">
                    <div id="pwgc-balance-amount-value"></div>
                </div>
                <div id="pwgc-balance-expiration-date">
                    <div id="pwgc-balance-expiration-date-value"></div>
                </div>

                <div id="pwgc-balance-activity"></div>
            </div>
        </div>
    </form>

    <?php
        require( 'footer.php' );
    ?>
</div>
